@@ -151,10 +151,13 @@ static void show_help(FishApplet* fish, const char* link_id)
 		uri = g_strdup_printf ("help:%s/%s", FISH_HELP_DOC, link_id);
 	else
 		uri = g_strdup_printf ("help:%s", FISH_HELP_DOC);
-
+#if GTK_CHECK_VERSION (3, 22, 0)
+	gtk_show_uri_on_window (NULL, uri, 
+			  gtk_get_current_event_time (), &error);
+#else
 	gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (fish)), uri,
 		      gtk_get_current_event_time (), &error);
-
+#endif
 	g_free (uri);
 
 	if (error &&
@@ -810,8 +813,9 @@ static void display_fortune_dialog(FishApplet* fish)
 	const char  *charset;
 	int          argc;
 	char       **argv;
-	GdkScreen *screen;
-	char *display;
+	GdkDisplay  *display;
+	GdkScreen   *screen;
+	char        *display_name;
 
 	/* if there is still a pipe, close it */
 	if (fish->source_id)
@@ -852,8 +856,8 @@ static void display_fortune_dialog(FishApplet* fish)
 
 		screen = gtk_widget_get_screen (GTK_WIDGET (fish));
 
-		screen_width  = gdk_screen_get_width (screen);
-		screen_height = gdk_screen_get_height (screen);
+		gdk_window_get_geometry (gdk_screen_get_root_window (screen), NULL, NULL,
+					 &screen_width, &screen_height);
 
 		gtk_window_set_default_size (GTK_WINDOW (fish->fortune_dialog),
 					     MIN (600, screen_width  * 0.9),
@@ -935,19 +939,20 @@ static void display_fortune_dialog(FishApplet* fish)
 	clear_fortune_text (fish);
 
 	screen = gtk_widget_get_screen (GTK_WIDGET (fish));
-	display = gdk_screen_make_display_name (screen);
+	display = gdk_screen_get_display (screen);
+	display_name = g_strdup (gdk_display_get_name (display));
 	g_spawn_async_with_pipes (NULL, /* working directory */
 							  argv,
 							  NULL, /* envp */
 							  G_SPAWN_SEARCH_PATH|G_SPAWN_STDERR_TO_DEV_NULL,
 							  set_environment,
-							  &display,
+							  &display_name,
 							  NULL, /* child pid */
 							  NULL, /* stdin */
 							  &output,
 							  NULL, /* stderr */
 							  &error);
-	g_free (display);
+	g_free (display_name);
 
 	if (error) {
 		char *message;
@@ -990,6 +995,9 @@ static void display_fortune_dialog(FishApplet* fish)
 
 static void name_changed_notify(GSettings* settings, gchar* key, FishApplet* fish)
 {
+	if (fish->source_id == 0)
+		return;
+
 	char *value;
 
 	value = g_settings_get_string (settings, key);
@@ -1015,6 +1023,9 @@ static void name_changed_notify(GSettings* settings, gchar* key, FishApplet* fis
 
 static void image_changed_notify(GSettings* settings, gchar* key, FishApplet* fish)
 {
+	if (fish->source_id == 0)
+		return;
+
 	char *value;
 
 	value = g_settings_get_string (settings, key);
@@ -1331,9 +1342,17 @@ static gboolean load_fish_image(FishApplet* fish)
 	return TRUE;
 }
 
+static gboolean
+update_pixmap_in_idle (gpointer data)
+{
+	update_pixmap (FISH_APPLET (data));
+	return FALSE;
+}
+
 static void update_pixmap(FishApplet* fish)
 {
 	GtkWidget     *widget = fish->drawing_area;
+	GtkRequisition prev_requisition;
 	GtkAllocation  allocation;
 	int            width  = -1;
 	int            height = -1;
@@ -1362,6 +1381,8 @@ static void update_pixmap(FishApplet* fish)
 	pixbuf_width  = gdk_pixbuf_get_width  (fish->pixbuf);
 	pixbuf_height = gdk_pixbuf_get_height (fish->pixbuf);
 
+	prev_requisition = fish->requisition;
+
 	if (fish->orientation == MATE_PANEL_APPLET_ORIENT_UP ||
 	    fish->orientation == MATE_PANEL_APPLET_ORIENT_DOWN) {
 		height = allocation.height;
@@ -1372,7 +1393,7 @@ static void update_pixmap(FishApplet* fish)
 		if (!rotate) {
 			width = allocation.width * fish->n_frames;
 			height = pixbuf_height * ((gdouble) width / pixbuf_width);
-			fish->requisition.width = width;
+			fish->requisition.width = allocation.width;
 			fish->requisition.height = height;
 		} else {
 			width = allocation.width;
@@ -1382,9 +1403,12 @@ static void update_pixmap(FishApplet* fish)
 		}
 	}
 
-	gtk_widget_set_size_request (fish->drawing_area,
-								 fish->requisition.width,
-								 fish->requisition.height);
+	if (prev_requisition.width  != fish->requisition.width ||
+	    prev_requisition.height != fish->requisition.height) {
+		gtk_widget_set_size_request (widget,
+					     fish->requisition.width,
+					     fish->requisition.height);
+		}
 
 	g_assert (width != -1 && height != -1);
 
@@ -1482,15 +1506,12 @@ static gboolean fish_applet_draw(GtkWidget* widget, cairo_t *cr, FishApplet* fis
 
 static void fish_applet_size_allocate(GtkWidget* widget, GtkAllocation* allocation, FishApplet* fish)
 {
-	GtkAllocation widget_allocation;
-
-	gtk_widget_get_allocation (widget, &widget_allocation);
-
-	if (widget_allocation.width  != fish->prev_allocation.width ||
-	    widget_allocation.height != fish->prev_allocation.height)
-		update_pixmap (fish);
+	if (allocation->width  == fish->prev_allocation.width &&
+	    allocation->height == fish->prev_allocation.height)
+		return;
 
 	fish->prev_allocation = *allocation;
+	g_idle_add (update_pixmap_in_idle, fish);
 }
 
 static void fish_applet_realize(GtkWidget* widget, FishApplet* fish)
@@ -1757,10 +1778,6 @@ static void fish_applet_dispose (GObject *object)
 {
 	FishApplet* fish = (FishApplet*) object;
 
-	if (fish->settings != NULL)
-		g_signal_handlers_disconnect_by_data (fish->settings,
-					  fish);
-
 	if (fish->timeout)
 	{
 		g_source_remove (fish->timeout);
