@@ -64,22 +64,15 @@ static GfStatusNotifierWatcher *sn_watcher_service = NULL;
 static GfStatusNotifierWatcher *
 sn_watcher_service_ref (void)
 {
-  GSettings *settings;
-  settings = g_settings_new ("org.mate.panel");
-
-  if (g_settings_get_boolean (settings, "enable-sni-support") == TRUE)
+  if (sn_watcher_service != NULL)
+    g_object_ref (sn_watcher_service);
+  else
     {
-      if (sn_watcher_service != NULL)
-        g_object_ref (sn_watcher_service);
-      else
-        {
-          sn_watcher_service = gf_status_notifier_watcher_new ();
-          g_object_add_weak_pointer ((GObject *) sn_watcher_service,
-                                     (gpointer *) &sn_watcher_service);
-        }
+      sn_watcher_service = gf_status_notifier_watcher_new ();
+      g_object_add_weak_pointer ((GObject *) sn_watcher_service,
+                                 (gpointer *) &sn_watcher_service);
     }
 
-  g_object_unref (settings);
   return sn_watcher_service;
 }
 #endif
@@ -111,9 +104,11 @@ static void help_cb(GtkAction* action, NaTrayApplet* applet)
 	#define NA_HELP_DOC "mate-user-guide"
 
 	uri = g_strdup_printf("help:%s/%s", NA_HELP_DOC, "panels-notification-area");
-
+#if GTK_CHECK_VERSION (3, 22, 0)
+	gtk_show_uri_on_window (NULL, uri, gtk_get_current_event_time (), &error);
+#else
 	gtk_show_uri(gtk_widget_get_screen(GTK_WIDGET(applet)), uri, gtk_get_current_event_time(), &error);
-
+#endif
 	g_free(uri);
 
 	if (error && g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
