@@ -30,8 +30,9 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libmatewnck/screen.h>
-#include <libmatewnck/util.h>
+#include <gdk/gdkx.h>
+#define WNCK_I_KNOW_THIS_IS_UNSTABLE
+#include <libwnck/libwnck.h>
 
 #include "wncklet.h"
 #include "window-menu.h"
@@ -45,12 +46,14 @@ void wncklet_display_help(GtkWidget* widget, const char* doc_id, const char* lin
 	char* uri;
 
 	if (link_id)
-		uri = g_strdup_printf("ghelp:%s?%s", doc_id, link_id);
+		uri = g_strdup_printf("help:%s/%s", doc_id, link_id);
 	else
-		uri = g_strdup_printf("ghelp:%s", doc_id);
-
+		uri = g_strdup_printf("help:%s", doc_id);
+#if GTK_CHECK_VERSION (3, 22, 0)
+	gtk_show_uri_on_window (NULL, uri, gtk_get_current_event_time (), &error);
+#else
 	gtk_show_uri(gtk_widget_get_screen(widget), uri, gtk_get_current_event_time(), &error);
-
+#endif
 	g_free(uri);
 
 	if (error && g_error_matches(error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
@@ -92,16 +95,16 @@ void wncklet_display_help(GtkWidget* widget, const char* doc_id, const char* lin
 	}
 }
 
-MatewnckScreen* wncklet_get_screen(GtkWidget* applet)
+WnckScreen* wncklet_get_screen(GtkWidget* applet)
 {
 	int screen_num;
 
 	if (!gtk_widget_has_screen(applet))
-		return matewnck_screen_get_default();
+		return wnck_screen_get_default();
 
-	screen_num = gdk_screen_get_number(gtk_widget_get_screen(applet));
+	screen_num = gdk_x11_screen_get_screen_number(gtk_widget_get_screen(applet));
 
-	return matewnck_screen_get(screen_num);
+	return wnck_screen_get(screen_num);
 }
 
 void wncklet_connect_while_alive(gpointer object, const char* signal, GCallback func, gpointer func_data, gpointer alive_object)
@@ -120,7 +123,7 @@ static gboolean wncklet_factory(MatePanelApplet* applet, const char* iid, gpoint
 
 	if (!type_registered)
 	{
-		matewnck_set_client_type(MATEWNCK_CLIENT_TYPE_PAGER);
+		wnck_set_client_type(WNCK_CLIENT_TYPE_PAGER);
 		type_registered = TRUE;
 	}
 
