@@ -23,6 +23,7 @@
 #include <string.h>
 #include <gtk/gtk.h>
 #include <gtk/gtkx.h>
+#include <panel-applets-manager.h>
 #include "panel-applet-container.h"
 #include "panel-marshal.h"
 
@@ -32,7 +33,10 @@ struct _MatePanelAppletContainerPrivate {
 	guint       name_watcher_id;
 	gchar      *bus_name;
 
+	gchar      *iid;
+	gboolean    out_of_process;
 	guint32     xid;
+	guint32     uid;
 	GtkWidget  *socket;
 
 	GHashTable *pending_ops;
@@ -86,19 +90,34 @@ static void mate_panel_applet_container_init(MatePanelAppletContainer* container
 {
 	container->priv = MATE_PANEL_APPLET_CONTAINER_GET_PRIVATE (container);
 
-	container->priv->socket = gtk_socket_new ();
-	g_signal_connect_swapped (container->priv->socket, "plug-removed", G_CALLBACK (mate_panel_applet_container_plug_removed), container);
-
 	container->priv->pending_ops = g_hash_table_new_full (g_direct_hash,
 							      g_direct_equal,
 							      NULL,
 							      (GDestroyNotify) g_object_unref);
-
-	gtk_container_add (GTK_CONTAINER (container),
-			   container->priv->socket);
-	gtk_widget_show (container->priv->socket);
 }
 
+static void
+panel_applet_container_setup (MatePanelAppletContainer *container)
+{
+	if (container->priv->out_of_process) {
+		container->priv->socket = gtk_socket_new ();
+
+		g_signal_connect_swapped (container->priv->socket,
+		                          "plug-removed",
+		                          G_CALLBACK (mate_panel_applet_container_plug_removed),
+		                          container);
+
+		gtk_container_add (GTK_CONTAINER (container), container->priv->socket);
+		gtk_widget_show (container->priv->socket);
+	} else {
+		GtkWidget *applet;
+
+		applet = mate_panel_applets_manager_get_applet_widget (container->priv->iid, container->priv->uid);
+
+		gtk_container_add (GTK_CONTAINER (container), applet);
+	}
+ }
+
 static void
 mate_panel_applet_container_cancel_pending_operations (MatePanelAppletContainer *container)
 {
@@ -133,6 +152,11 @@ mate_panel_applet_container_dispose (GObject *object)
 		container->priv->bus_name = NULL;
 	}
 
+	if (container->priv->iid) {
+		g_free (container->priv->iid);
+		container->priv->iid = NULL;
+	}
+
 	if (container->priv->name_watcher_id > 0) {
 		g_bus_unwatch_name (container->priv->name_watcher_id);
 		container->priv->name_watcher_id = 0;
@@ -347,6 +371,8 @@ on_proxy_appeared (GObject      *source_object,
 	g_simple_async_result_complete (result);
 	g_object_unref (result);
 
+	panel_applet_container_setup (container);
+
 	if (container->priv->xid > 0) {
 		gtk_socket_add_id (GTK_SOCKET (container->priv->socket),
 				   container->priv->xid);
@@ -379,7 +405,12 @@ get_applet_cb (GObject      *source_object,
 	}
 
 	container = MATE_PANEL_APPLET_CONTAINER (g_async_result_get_source_object (G_ASYNC_RESULT (result)));
-	g_variant_get (retvals, "(&ou)", &applet_path, &container->priv->xid);
+	g_variant_get (retvals,
+	               "(&obuu)",
+	               &applet_path,
+	               &container->priv->out_of_process,
+	               &container->priv->xid,
+	               &container->priv->uid);
 
 	g_dbus_proxy_new (connection,
 			  G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES,
@@ -432,7 +463,7 @@ on_factory_appeared (GDBusConnection   *connection,
 				MATE_PANEL_APPLET_FACTORY_INTERFACE,
 				"GetApplet",
 				data->parameters,
-				G_VARIANT_TYPE ("(ou)"),
+				G_VARIANT_TYPE ("(obuu)"),
 				G_DBUS_CALL_FLAGS_NONE,
 				-1,
 				data->cancellable,
@@ -479,7 +510,7 @@ mate_panel_applet_container_get_applet (MatePanelAppletContainer *container,
 
 	/* we can't use the screen of the container widget since it's not in a
 	 * widget hierarchy yet */
-	screen_number = gdk_screen_get_number (screen);
+	screen_number = gdk_x11_screen_get_screen_number (screen);
 
 	data = g_new (AppletFactoryData, 1);
 	data->result = result;
@@ -489,6 +520,7 @@ mate_panel_applet_container_get_applet (MatePanelAppletContainer *container,
 
 	bus_name = g_strdup_printf (MATE_PANEL_APPLET_BUS_NAME, factory_id);
 
+	container->priv->iid = g_strdup (iid);
 	container->priv->name_watcher_id =
 		g_bus_watch_name (G_BUS_TYPE_SESSION,
 				  bus_name,
@@ -562,7 +594,7 @@ set_applet_property_cb (GObject      *source_object,
 	g_object_unref (container);
 }
 
-gconstpointer
+void
 mate_panel_applet_container_child_set (MatePanelAppletContainer *container,
 				  const gchar          *property_name,
 				  const GVariant       *value,
@@ -575,7 +607,7 @@ mate_panel_applet_container_child_set (MatePanelAppletContainer *container,
 	GSimpleAsyncResult       *result;
 
 	if (!proxy)
-		return NULL;
+		return;
 
 	info = mate_panel_applet_container_child_property_get_info (property_name);
 	if (!info) {
@@ -585,7 +617,7 @@ mate_panel_applet_container_child_set (MatePanelAppletContainer *container,
 						     MATE_PANEL_APPLET_CONTAINER_INVALID_CHILD_PROPERTY,
 						     "%s: Applet has no child property named `%s'",
 						     G_STRLOC, property_name);
-		return NULL;
+		return;
 	}
 
 	result = g_simple_async_result_new (G_OBJECT (container),
@@ -613,8 +645,6 @@ mate_panel_applet_container_child_set (MatePanelAppletContainer *container,
 				-1, cancellable,
 				set_applet_property_cb,
 				result);
-
-	return result;
 }
 
 gboolean
@@ -666,7 +696,7 @@ get_applet_property_cb (GObject      *source_object,
 	g_object_unref (container);
 }
 
-gconstpointer
+void
 mate_panel_applet_container_child_get (MatePanelAppletContainer *container,
 				  const gchar          *property_name,
 				  GCancellable         *cancellable,
@@ -678,7 +708,7 @@ mate_panel_applet_container_child_get (MatePanelAppletContainer *container,
 	GSimpleAsyncResult       *result;
 
 	if (!proxy)
-		return NULL;
+		return;
 
 	info = mate_panel_applet_container_child_property_get_info (property_name);
 	if (!info) {
@@ -688,7 +718,7 @@ mate_panel_applet_container_child_get (MatePanelAppletContainer *container,
 						     MATE_PANEL_APPLET_CONTAINER_INVALID_CHILD_PROPERTY,
 						     "%s: Applet has no child property named `%s'",
 						     G_STRLOC, property_name);
-		return NULL;
+		return;
 	}
 
 	result = g_simple_async_result_new (G_OBJECT (container),
@@ -714,8 +744,6 @@ mate_panel_applet_container_child_get (MatePanelAppletContainer *container,
 				-1, cancellable,
 				get_applet_property_cb,
 				result);
-
-	return result;
 }
 
 GVariant *
@@ -798,14 +826,3 @@ mate_panel_applet_container_child_popup_menu_finish (MatePanelAppletContainer *c
 
 	return !g_simple_async_result_propagate_error (simple, error);
 }
-
-void
-mate_panel_applet_container_cancel_operation (MatePanelAppletContainer *container,
-                                              gconstpointer             operation)
-{
-	gpointer value = g_hash_table_lookup (container->priv->pending_ops, operation);
-	if (value == NULL)
-		return;
-
-	g_cancellable_cancel (G_CANCELLABLE (value));
-}
