@@ -202,11 +202,19 @@ panel_multiscreen_get_gdk_monitors_for_screen (GdkScreen     *screen,
 					       int           *monitors_ret,
 					       GdkRectangle **geometries_ret)
 {
+#if GTK_CHECK_VERSION (3, 22, 0)
+	GdkDisplay   *display;
+#endif
 	int           num_monitors;
 	GdkRectangle *geometries;
 	int           i;
 
+#if GTK_CHECK_VERSION (3, 22, 0)
+	display = gdk_screen_get_display (screen);
+	num_monitors = gdk_display_get_n_monitors (display);
+#else
 	num_monitors = gdk_screen_get_n_monitors (screen);
+#endif
 	geometries = g_new (GdkRectangle, num_monitors);
 
 	for (i = 0; i < num_monitors; i++)
@@ -404,7 +412,6 @@ void
 panel_multiscreen_init (void)
 {
 	GdkDisplay *display;
-	int         i;
 
 	if (initialized)
 		return;
@@ -417,24 +424,22 @@ panel_multiscreen_init (void)
 	monitors   = g_new0 (int, screens);
 	geometries = g_new0 (GdkRectangle *, screens);
 
-	for (i = 0; i < screens; i++) {
-		GdkScreen *screen;
+	GdkScreen *screen;
 
-		screen = gdk_display_get_screen (display, i);
+	screen = gdk_display_get_default_screen (display);
 
-		/* We connect to both signals to be on the safe side, but in
-		 * theory, it should be enough to only connect to
-		 * monitors-changed. Since we'll likely get two signals, we do
-		 * the real callback in the idle loop. */
-		g_signal_connect (screen, "size-changed",
-				  G_CALLBACK (panel_multiscreen_queue_reinit), NULL);
-		g_signal_connect (screen, "monitors-changed",
-				  G_CALLBACK (panel_multiscreen_queue_reinit), NULL);
+	/* We connect to both signals to be on the safe side, but in
+	 * theory, it should be enough to only connect to
+	 * monitors-changed. Since we'll likely get two signals, we do
+	 * the real callback in the idle loop. */
+	g_signal_connect (screen, "size-changed",
+			  G_CALLBACK (panel_multiscreen_queue_reinit), NULL);
+	g_signal_connect (screen, "monitors-changed",
+			  G_CALLBACK (panel_multiscreen_queue_reinit), NULL);
 
-		panel_multiscreen_get_monitors_for_screen (screen,
-							   &(monitors[i]),
-							   &(geometries[i]));
-	}
+	panel_multiscreen_get_monitors_for_screen (screen,
+						   &(monitors[0]),
+						   &(geometries[0]));
 
 	initialized = TRUE;
 }
@@ -481,7 +486,7 @@ panel_multiscreen_monitors (GdkScreen *screen)
 {
 	int n_screen;
 
-	n_screen = gdk_screen_get_number (screen);
+	n_screen = gdk_x11_screen_get_screen_number (screen);
 
 	g_return_val_if_fail (n_screen >= 0 && n_screen < screens, 1);
 
@@ -494,7 +499,7 @@ panel_multiscreen_x (GdkScreen *screen,
 {
 	int n_screen;
 
-	n_screen = gdk_screen_get_number (screen);
+	n_screen = gdk_x11_screen_get_screen_number (screen);
 
 	g_return_val_if_fail (n_screen >= 0 && n_screen < screens, 0);
 	g_return_val_if_fail (monitor >= 0 && monitor < monitors [n_screen], 0);
@@ -508,7 +513,7 @@ panel_multiscreen_y (GdkScreen *screen,
 {
 	int n_screen;
 
-	n_screen = gdk_screen_get_number (screen);
+	n_screen = gdk_x11_screen_get_screen_number (screen);
 
 	g_return_val_if_fail (n_screen >= 0 && n_screen < screens, 0);
 	g_return_val_if_fail (monitor >= 0 && monitor < monitors [n_screen], 0);
@@ -522,7 +527,7 @@ panel_multiscreen_width (GdkScreen *screen,
 {
 	int n_screen;
 
-	n_screen = gdk_screen_get_number (screen);
+	n_screen = gdk_x11_screen_get_screen_number (screen);
 
 	g_return_val_if_fail (n_screen >= 0 && n_screen < screens, 0);
 	g_return_val_if_fail (monitor >= 0 && monitor < monitors [n_screen], 0);
@@ -536,7 +541,7 @@ panel_multiscreen_height (GdkScreen *screen,
 {
 	int n_screen;
 
-	n_screen = gdk_screen_get_number (screen);
+	n_screen = gdk_x11_screen_get_screen_number (screen);
 
 	g_return_val_if_fail (n_screen >= 0 && n_screen < screens, 0);
 	g_return_val_if_fail (monitor >= 0 && monitor < monitors [n_screen], 0);
@@ -588,7 +593,7 @@ panel_multiscreen_get_monitor_at_point (GdkScreen *screen,
 	/* not -1 as callers expect a real monitor */
 	g_return_val_if_fail (GDK_IS_SCREEN (screen), 0);
 
-	n_screen = gdk_screen_get_number (screen);
+	n_screen = gdk_x11_screen_get_screen_number (screen);
 
 	n_monitors = monitors[n_screen];
 	geoms = geometries[n_screen];
@@ -653,7 +658,7 @@ panel_multiscreen_is_at_visible_extreme (GdkScreen *screen,
 	MonitorBounds monitor;
 	int           n_screen, i;
 
-	n_screen = gdk_screen_get_number (screen);
+	n_screen = gdk_x11_screen_get_screen_number (screen);
 
 	*leftmost   = TRUE;
 	*rightmost  = TRUE;
